# -*- coding: utf-8 -*-
"""
Created on Thu May 13 11:59:29 2021

@author: bgregor
"""

# Count letter frequency in 1M words

 
# The multiprocessing library
import multiprocessing as mp
import string

# A function to download a file of 1M words
from io import BytesIO
from zipfile import ZipFile
from urllib.request import urlopen
import itertools as its
import pprint

from get_n_cores import get_n_cores


def get_data():
    ''' Fetches a ZIP file, unpacks it to a file in the temp directory,
        returns the opened file. The data file is from SuperFastPython,
        an extensive tutorial on using multiprocessing:
            https://superfastpython.com/category/multiprocessing/
    '''
    resp = urlopen('https://raw.githubusercontent.com/SuperFastPython/DataSets/main/bin/1m_words.txt.zip')
    myzip = ZipFile(BytesIO(resp.read()))
    all_lines = [word.decode('utf-8') for word in myzip.open('1m_words.txt').readlines()]
    # Strip and lowercase the strings to remove newline characters
    return [word.strip().lower() for word in all_lines]
    
def count_letters(words):
    ''' Return a dictionary of every lowercase letter
    in English found in a list of words'''
    print(f'len words: {len(words)}')
    # hint: here's a built-in Python list of the English alphabet
    alphabet = list(string.ascii_lowercase)
    # and here's how to initialize a dictionary of {'a':0,'b':0,...}
    # using the itertools library without a 'for' loop. dict() will
    # create a dictionary from a list of (key,value) tuples
    letter_counts = dict(zip(alphabet,its.repeat(0)))
    # and fill it in...
    for word in words:
        for w in word:
            if w in letter_counts: # check in case w is punctuation
                letter_counts[w] += 1 
    return letter_counts    


def par_count_letters(words, ncores):
    ''' A parallel version of count_letters '''
    with mp.Pool(processes=ncores) as pool:
    # How to parallelize with the pool?  Can you 
    # re-use count_letters or do you need some variation 
    # of it?
    # This next line is just a placeholder, delete it
    # when you start implementing this function:
        letter_count = pool.map(count_letters,...)
    return letter_count
    

if __name__ == '__main__':
    # Fetch the data.
    words = get_data()
    # Print out the dictionary of letter counts:
    letter_counts = count_letters(words)
    pp = pprint.PrettyPrinter(indent=4)
    pp.pprint(letter_counts)    

    # Now do it again in parallel...
    ncores = get_n_cores()
    letter_counts_par = par_count_letters(words, ncores)
    # and print it - the results should be the same!
    pp.pprint(letter_counts_par)    
